// ======================================================================
//
// gl_abstract.h
// jeff grills
//
// copyright 1998 origin systems inc
//
// ======================================================================

#ifndef GL_ABSTRACT_H
#define GL_ABSTRACT_H

// ======================================================================

class  Camera;
class  Bitmap;
class  StaticMaterial;
class  MaterialData;
class  StaticMaterial;
class  Light;
class  Palette;
class  PaletteData;
class  Vector;
class  Xform;
class  ClipInfo;

struct GL_input;
struct GL_output;
struct uv_instance;
struct UVertex;
struct TVertex;

#include "Gl.def"

// ======================================================================

class GL_abstract
{
public:

	enum
	{
		// Increment this number whenever you change the anything in this class
		MAJOR = 4
	};

private:

	// disable copy constructor and assignment operator
	GL_abstract(const GL_abstract &);
	GL_abstract &operator =(const GL_abstract &);
	
public:

	struct Information
	{
		const char *name;
		int         versionMajor;
		int         versionMinor;
		int         ranking;
	};

protected:

	// frame buffer information
	static int               width;
	static int               height;
	static int               depth;

	// screen background clearing information
	static const StaticMaterial   *backgroundMaterial;

	// viewport clip information
 	static const ClipInfo  **clipInfo;

	// screen shot file name
	static char             *screenShotName;

public:

	static int              getWidth(void);
	static int              getHeight(void);
	static int              getDepth(void);

	static const ClipInfo  &getClipInfo(void);

	void (*submitPointListFunction)(const TVertex *vertexList, int numberOfVertices);
	void (*submitLineListFunction)(const TVertex *vertexList, int numberOfVertices);
	void (*submitLineStripFunction)(const TVertex *vertexList, int numberOfVertices);
	void (*submitTriangleListFunction)(const TVertex *vertexList, int numberOfVertices);
	void (*submitTriangleStripFunction)(const TVertex *vertexList, int numberOfVertices);
	void (*submitTriangleFanFunction)(const TVertex *vertexList, int numberOfVertices);

private:

	static void defaultSubmitFunction(const TVertex *vertexList, int numberOfVertices);

public:

	GL_abstract(void);
	virtual ~GL_abstract(void);

	// version information
	virtual void  getInformation(Information *information) = 0;

	// query to see if the rasterizer can be used
	virtual bool  query(int minor) = 0;

	// initialize the rasterizer
	virtual bool  init(const GL_input *input, GL_output *output);

	// dump debug informtion
	virtual void  debugDump(void);

	// set the output window
	virtual void  setDrawWindow(HWND newDrawWindow);

	// application activation
	virtual void  setActive(bool active) = 0;

	// access functions
	virtual bool  lock(void) = 0;
	virtual void  unlock(bool display) = 0;
	virtual void  screenShot(const char *fileName);
	virtual bool  gdiVisible(void) = 0;

	// set processing of primitives
	virtual void setAlphaBlendEnabled(bool enabled) = 0;
	virtual void setAlphaBlendFunctions(AlphaBlendFunction source, AlphaBlendFunction destination) = 0;
	virtual void setPrimitiveProcessing(PrimitiveProcessing primitiveProcessing) = 0;

	// palette data functions
	virtual PaletteData  *paletteCreateData(const Palette *pal) = 0;
	virtual void          paletteDestroyData(PaletteData *palData) = 0;

	// material data functions
	virtual MaterialData *materialCreateData(const StaticMaterial *material) = 0;
	virtual void          materialDestroyData(MaterialData *mat) = 0;

	// material functions
	virtual void          setBackgroundMaterial(const StaticMaterial *material);
	virtual void          setMaterial(const StaticMaterial *material) = 0;

	// camera render functions
	virtual void beginRender(const Camera &camera) = 0;
	virtual void endRender(void) = 0;

	// 3d setup functions
	virtual void setLightingEnabled(bool lightingEnabled) = 0;
	virtual void setLightList(Light **lightList, int lightCount) = 0;
	virtual bool setVertexList(const Vector *vertex, int vertexCount, const Vector *normal, int normalCount) = 0;
	virtual void setObjectToWorld(const Xform &objectToWorld) = 0;

	// object-space primitives
	virtual void drawPointList(const UVertex *vertexList, int numberOfVertices) = 0;
	virtual void drawLineList(const UVertex *vertexList, int numberOfVertices) = 0;
	virtual void drawLineStrip(const UVertex *vertexList, int numberOfVertices) = 0;
	virtual void drawTriangleList(const UVertex *vertexList, int numberOfVertices) = 0;
	virtual void drawTriangleStrip(const UVertex *vertexList, int numberOfVertices) = 0;
	virtual void drawTriangleFan(const UVertex *vertexList, int numberOfVertices) = 0;

	// screen-space primitives
	virtual void drawPointList(const TVertex *vertexList, int numberOfVertices) = 0;
	virtual void drawLineList(const TVertex *vertexList, int numberOfVertices) = 0;
	virtual void drawLineStrip(const TVertex *vertexList, int numberOfVertices) = 0;
	virtual void drawTriangleList(const TVertex *vertexList, int numberOfVertices) = 0;
	virtual void drawTriangleStrip(const TVertex *vertexList, int numberOfVertices) = 0;
	virtual void drawTriangleFan(const TVertex *vertexList, int numberOfVertices) = 0;

	// submit transformed/clipped primitives to the rasterizer
	void submitPointList(const TVertex *vertexList, int numberOfVertices);
	void submitLineList(const TVertex *vertexList, int numberOfVertices);
	void submitLineStrip(const TVertex *vertexList, int numberOfVertices);
	void submitTriangleList(const TVertex *vertexList, int numberOfVertices);
	void submitTriangleStrip(const TVertex *vertexList, int numberOfVertices);
	void submitTriangleFan(const TVertex *vertexList, int numberOfVertices);

	// movie primitive
	virtual void  movie(const uv_instance *uv);

	// per-rasterizer special effects
	virtual bool  specialEffect(int id, void *data);
};

// ======================================================================

inline int GL_abstract::getWidth(void)
{
	return width;
}

// ----------------------------------------------------------------------

inline int GL_abstract::getHeight(void)
{
	return height;
}

// ----------------------------------------------------------------------

inline int GL_abstract::getDepth(void)
{
	return depth;
}

// ----------------------------------------------------------------------

inline const ClipInfo &GL_abstract::getClipInfo(void)
{
	return **clipInfo;
}

// ----------------------------------------------------------------------

inline void GL_abstract::submitPointList(const TVertex *vertexList, int numberOfVertices)
{
	submitPointListFunction(vertexList, numberOfVertices);
}

// ----------------------------------------------------------------------

inline void GL_abstract::submitLineList(const TVertex *vertexList, int numberOfVertices)
{
	submitLineListFunction(vertexList, numberOfVertices);
}

// ----------------------------------------------------------------------

inline void GL_abstract::submitLineStrip(const TVertex *vertexList, int numberOfVertices)
{
	submitLineStripFunction(vertexList, numberOfVertices);
}

// ----------------------------------------------------------------------

inline void GL_abstract::submitTriangleList(const TVertex *vertexList, int numberOfVertices)
{
	submitTriangleListFunction(vertexList, numberOfVertices);
}

// ----------------------------------------------------------------------

inline void GL_abstract::submitTriangleStrip(const TVertex *vertexList, int numberOfVertices)
{
	submitTriangleStripFunction(vertexList, numberOfVertices);
}

// ----------------------------------------------------------------------

inline void GL_abstract::submitTriangleFan(const TVertex *vertexList, int numberOfVertices)
{
	submitTriangleFanFunction(vertexList, numberOfVertices);
}

// ======================================================================

#endif
